import operator

import sfml as sf

import bus
from tools import *

NODE_ID = 4

moving_average = [None for i in range(8)]
    
def load_calibration():
    ''' Loads calibration tables and stores them as global variables.'''
    global calibCW, calibCCW, calibSmall
    
    calibration = pickle.load(open('calibration.pkl','rb'))
    calibCW = [[c[2], c[0]] for c in calibration[:len(calibration)/2 + 1]]
    calibCCW = [[c[2], c[0]] for c in calibration[len(calibration)/2:]] + \
                   [[calibration[0][2], 2*pi]]
    calibSmall = [[c[0], c[1]] for c in calibration] + [[2*pi, calibration[0][1]]]

def get_angle():
    ''' Polls the device for a new set of values; appends them to the
        moving average list.  Returns the average angle from the moving
        average list.'''
    global moving_average
    global calibCW, calibCCW, calibSmall
    
    line = bus.sample(NODE_ID)
    small, large = [int(v) for v in line[:-1].split('\t')]
    
    # Project forward to figure out which angle is most likely    
    angleCW = interpolate(large, calibCW)
    angleCCW = interpolate(large, calibCCW)

    errorCW  = abs(interpolate(angleCW,  calibSmall) - small)
    errorCCW = abs(interpolate(angleCCW, calibSmall) - small)

    this_angle = angleCW if errorCW < errorCCW else angleCCW
    moving_average = moving_average[1:] + [this_angle]
    
    angles = filter(lambda a: a is not None, moving_average)
    projection = reduce(
        operator.add,
        [sf.Vector2f(sin(a), cos(a)) for a in angles])
    
    return atan2(projection.y, projection.x)

load_calibration()