import cPickle as pickle
from math import sqrt, sin, cos, atan2, pi

def mean(values):
    ''' Returns the average of a list of values. '''
    values = filter(lambda v: v is not None, values)
    return sum(values) / float(len(values))

def interpolate(reading, data):
    '''Takes in a single reading and a list of data tuples in the
       order (reading, value).  Returns a first-order interpolation
       mapping the input reading to value.
       
       The sequence data[1 to n][0] must be monotonically increasing
       or decreasing for interpolation to work correctly.'''
       
    # The provided data is either ascending or descending.
    # If descending, flip so that it is ascending.
    if data[0][0] > data[1][0]:
        data = data[::-1]
        
    if reading < data[0][0]:
        i = 0
    elif reading > data[-1][0]:
        i = len(data) - 2
    else:
        for i in range(len(data) - 1):
            if reading >= data[i][0] and reading <= data[i+1][0]:
                break

    slope = float(data[i+1][1] - data[i][1]) \
               / (data[i+1][0] - data[i][0])
    return data[i][1] + slope*(reading - data[i][0])
    

def norm(vector):
    return sqrt(vector.x**2 + vector.y**2)